#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources import pipeline
from yagocd.resources import Base
import mock
import pytest


class TestPipelineEntity(object):
    @pytest.fixture()
    def pipeline_entity(self, mock_session):
        return pipeline.PipelineEntity(
            session=mock_session,
            data={'name': 'pipeline_1',
                  'materials': [
                      {'description': 'child_name_1', 'type': 'Pipeline'},
                      {'description': 'git_repo_1', 'type': 'Git'},
                      {'description': 'child_name_2', 'type': 'Pipeline'},
                  ]
                  },
            group='baz'
        )

    def test_entity_is_not_none(self, pipeline_entity):
        assert pipeline_entity is not None

    def test_is_instance_of_base(self, pipeline_entity):
        assert isinstance(pipeline_entity, Base)

    def test_reading_group(self, pipeline_entity):
        assert pipeline_entity.group == 'baz'

    def test_predecessors(self, pipeline_entity):
        assert pipeline_entity.predecessors == [
            {'description': 'child_name_1', 'type': 'Pipeline'},
            {'description': 'child_name_2', 'type': 'Pipeline'}
        ]

    def test_descendants_empty(self, pipeline_entity):
        assert pipeline_entity.descendants is None

    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_history_call(self, history_mock, pipeline_entity):
        pipeline_entity.history()
        history_mock.assert_called_with(name=pipeline_entity.data.name, offset=0)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.status')
    def test_status_call(self, status_mock, pipeline_entity):
        pipeline_entity.status()
        status_mock.assert_called_with(name=pipeline_entity.data.name)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.pause')
    def test_pause_call(self, pause_mock, pipeline_entity):
        pipeline_entity.pause('custom-reason')
        pause_mock.assert_called_with(name=pipeline_entity.data.name, cause='custom-reason')

    @mock.patch('yagocd.resources.pipeline.PipelineManager.unpause')
    def test_unpause_call(self, unpause_mock, pipeline_entity):
        pipeline_entity.unpause()
        unpause_mock.assert_called_with(name=pipeline_entity.data.name)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.release_lock')
    def test_release_lock_call(self, release_lock_mock, pipeline_entity):
        pipeline_entity.release_lock()
        release_lock_mock.assert_called_with(name=pipeline_entity.data.name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import os
import sys
import time
import subprocess

import mock
import pytest
import requests
from vcr import VCR

vcr_obj = VCR(
    cassette_library_dir='tests/fixtures/cassettes',
    path_transformer=VCR.ensure_suffix('.yaml')
)


@pytest.fixture()
def mock_session():
    return mock.patch('yagocd.session.Session').start()


@pytest.fixture()
def my_vcr():
    return vcr_obj


CONTAINER_NAME = 'yagocd-server'
CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))


def start_container():
    output = subprocess.check_output([
        "/usr/local/bin/docker",
        "ps",
        "--quiet",
        "--filter=name={container_name}".format(
            container_name=CONTAINER_NAME),
    ])

    if not output:
        subprocess.check_call([
            "/usr/local/bin/docker",
            "run",
            "--detach",
            "--net=host",
            "--volume={current_dir}/docker:/workspace".format(
                current_dir=CURRENT_DIR
            ),
            "--workdir=/workspace",
            "--name={container_name}".format(
                container_name=CONTAINER_NAME
            ),
            "gocd/gocd-dev",
            "/bin/bash",
            "-c",
            "'/workspace/bootstrap.sh'",
        ])


def wait_till_started():
    while True:
        try:
            requests.get("http://local.docker:8153/go")
        except requests.exceptions.ConnectionError:
            sys.stdout.write('.')
            time.sleep(5)
        else:
            break


def stop_container():
    subprocess.check_call([
        "/usr/local/bin/docker",
        "rm",
        "-f",
        CONTAINER_NAME
    ])


@pytest.fixture(scope="session")
def gocd_docker(request):
    start_container()
    wait_till_started()

    def fin():
        stop_container()

    request.addfinalizer(fin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from StringIO import StringIO
import csv

from yagocd.resources import BaseManager


class PropertyManager(BaseManager):
    """
    The properties API allows managing of job properties.
    """

    def __init__(
        self,
        session,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None
    ):
        """
        Constructs instance of ``PropertyManager``.
        Parameters to the constructor and methods of the class could be duplicated. That is because of two use cases
        of this class:
            1. When the class being instantiated from :class:`yagocd.client.Client`, we don't know all the necessary
             parameters yet, but we need an instance to work with. So we skip parameters instantiation in constructor,
             but require them for each method.
            2. When the class being used from :class:`yagocd.resources.job.JobInstance` - in this case we already
            know all required parameters, so we can instantiate `PropertyManager` with them.

        :param session: session object from client.
        :type session: yagocd.session.Session.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        """
        super(PropertyManager, self).__init__(session)

        self.base_api = self._session.base_api(api_path='')

        self._pipeline_name = pipeline_name
        self._pipeline_counter = pipeline_counter
        self._stage_name = stage_name
        self._stage_counter = stage_counter
        self._job_name = job_name

    def list(
        self,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None
    ):
        """
        Lists all job properties.

        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return: dictionary of properties.
        :rtype: dict[str, str]
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter
        assert self._job_name or job_name

        response = self._session.post(
            path='{base_api}/properties/{pipeline_name}/{pipeline_counter}/{stage_name}/{stage_counter}/{job_name}'.format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter,
                job_name=self._job_name or job_name
            ),
            headers={'Accept': 'application/json'},
        )
        text = StringIO(response.text)
        parsed = list(csv.reader(text))
        properties = dict(zip(parsed[0], parsed[1]))

        return properties

    def get(
        self,
        name,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None
    ):
        """
        Gets a property by its name.
        :info: You can use keyword `latest` as a pipeline counter or a stage counter.

        :param name: name of property to get.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return: single property as a dictionary.
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter
        assert self._job_name or job_name

        response = self._session.get(
            path='{base_api}/properties/{pipeline_name}/{pipeline_counter}/{stage_name}/{stage_counter}/{job_name}/{name}'.format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter,
                job_name=self._job_name or job_name,
                name=name
            ),
            headers={'Accept': 'application/json'},
        )
        text = StringIO(response.text)
        parsed = list(csv.reader(text))
        properties = dict(zip(parsed[0], parsed[1]))

        return properties

    def historical(self, pipeline_name=None, stage_name=None, job_name=None, limit_pipeline=None, limit_count=None):
        """
        Get historical properties.
        :info: `limitPipeline` and `limitCount` are optional parameters. The default value of
        `limitPipeline` is latest pipeline instance’s counter. The default value of `limitCount` is `100`.

        :param pipeline_name: name of the pipeline.
        :param stage_name: name of the stage.
        :param job_name: name of the job.
        :param limit_pipeline: pipeline limit for returned properties.
        :param limit_count: count limit for returned properties.
        :return: list of dictionaries as historical values.
        """
        assert self._pipeline_name or pipeline_name
        assert self._stage_name or stage_name
        assert self._job_name or job_name

        params = {
            'pipelineName': self._pipeline_name or pipeline_name,
            'stageName': self._stage_name or stage_name,
            'jobName': self._job_name or job_name,
        }
        if limit_pipeline is not None:
            params['limitPipeline'] = limit_pipeline
        if limit_count is not None:
            params['limitCount'] = limit_count

        response = self._session.get(
            path='{base_api}/properties/search'.format(base_api=self.base_api),
            params=params,
            headers={'Accept': 'application/json'},
        )

        text = StringIO(response.text)
        result = list(csv.DictReader(text))

        return result

    def create(
        self,
        name,
        value,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None
    ):
        """
        Defines a property on a specific job instance.

        :param name: name of property.
        :param value: value of property.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return: an acknowledgement that the property was created.
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter
        assert self._job_name or job_name

        response = self._session.post(
            path='{base_api}/properties/{pipeline_name}/{pipeline_counter}/{stage_name}/{stage_counter}/{job_name}/{name}'.format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter,
                job_name=self._job_name or job_name,
                name=name
            ),
            data={'value': value},
            headers={'Accept': 'application/json'},
        )
        return response.text


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources.job import JobInstance
from yagocd.resources import BaseManager, Base


class StageManager(BaseManager):
    """
    The stages API allows users to view stage information and operate on it.
    """

    def cancel(self, pipeline, stage):
        """
        Cancel an active stage of a specified stage.

        :param pipeline: pipeline name.
        :param stage: stage name.
        :return: a text confirmation.
        """
        response = self._session.post(
            path='{base_api}/stages/{pipeline}/{stage}/cancel'.format(
                base_api=self.base_api,
                pipeline=pipeline,
                stage=stage
            ),
            headers={'Accept': 'application/json'},
        )
        return response.text

    def get(self, pipeline, stage, pipeline_counter, stage_counter):
        """
        Gets stage instance object.

        :param pipeline: pipeline name.
        :param stage: stage name.
        :param pipeline_counter: pipeline counter.
        :param stage_counter: stage counter.
        :return: a stage instance object :class:`yagocd.resources.stage.StageInstance`.
        :rtype: yagocd.resources.stage.StageInstance
        """
        response = self._session.get(
            path='{base_api}/stages/{pipeline}/{stage}/instance/{pipeline_counter}/{stage_counter}'.format(
                base_api=self.base_api,
                pipeline=pipeline,
                stage=stage,
                pipeline_counter=pipeline_counter,
                stage_counter=stage_counter
            ),
            headers={'Accept': 'application/json'},
        )

        return StageInstance(session=self._session, data=response.json(), pipeline=None)

    def history(self, pipeline, stage, offset=0):
        """
        The stage history allows users to list stage instances of specified stage.
        Supports pagination using offset which tells the API how many instances to skip.

        :param pipeline: pipeline name.
        :param stage: stage name.
        :param offset: how many instances to skip.
        :return: an array of stage instances :class:`yagocd.resources.stage.StageInstance`.
        :rtype: list of yagocd.resources.stage.StageInstance
        """
        response = self._session.get(
            path='{base_api}/stages/{pipeline}/{stage}/history/{offset}'.format(
                base_api=self.base_api,
                pipeline=pipeline,
                stage=stage,
                offset=offset,
            ),
            headers={'Accept': 'application/json'},
        )

        instances = list()
        for instance in response.json().get('stages'):
            instances.append(StageInstance(session=self._session, data=instance, pipeline=None))

        return instances


class StageInstance(Base):
    """
    Class representing instance of specific stage.
    """

    def __init__(self, session, data, pipeline):
        super(StageInstance, self).__init__(session, data)
        self._pipeline = pipeline

    @property
    def url(self):
        """
        Returns url for accessing stage instance.
        """
        return "{server_url}/go/pipelines/{pipeline_name}/{pipeline_counter}/{stage_name}/{stage_counter}".format(
            server_url=self._session.server_url,
            pipeline_name=self._pipeline.data.name,
            pipeline_counter=self._pipeline.data.counter,
            stage_name=self.data.name,
            stage_counter=self.data.counter,
        )

    @property
    def pipeline(self):
        return self._pipeline

    def jobs(self):
        """
        Method for getting jobs from stage instance.

        :return: arrays of jobs.
        :rtype: list of yagocd.resources.job.JobInstance
        """
        jobs = list()
        for data in self.data.jobs:
            jobs.append(JobInstance(session=self._session, data=data, stage=self))

        return jobs


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import json
from yagocd.resources import BaseManager, Base


class UserManager(BaseManager):
    """
    The users API allows users with administrator role to manage users.
    """

    def list(self):
        """
        Lists all available users.
        :return: an array of user objects :class:`yagocd.resources.user.UserEntity`.
        :rtype: yagocd.resources.user.UserEntity
        """
        response = self._session.get(
            path='{base_api}/users'.format(base_api=self.base_api),
        )

        users = list()
        for data in response.json().get('_embedded', {}).get('users', {}):
            users.append(UserEntity(session=self._session, data=data))

        return users

    def get(self, login):
        """
        Gets a user by its login name.

        :param login: login name.
        :return: a user object :class:`yagocd.resources.user.UserEntity`.
        :rtype: yagocd.resources.user.UserEntity
        """
        response = self._session.get(
            path='{base_api}/users/{login}'.format(
                base_api=self.base_api,
                login=login
            ),
        )

        return UserEntity(session=self._session, data=response.json())

    def create(self, options):
        """
        Creates a user.

        Can't make this one work: returns 404 all the time.

        :param options: dictionary of parameters for creating a user.
        :return: a user object :class:`yagocd.resources.user.UserEntity`.
        :rtype: yagocd.resources.user.UserEntity
        """
        response = self._session.post(
            path='{base_api}/users'.format(
                base_api=self.base_api
            ),
            data=json.dumps(options),
            headers={
                'Content-Type': 'application/json'
            }
        )

        return response.text

    def update(self, login, options):
        """
        Update some attributes of a user.

        :param login: login name.
        :param options: dictionary of parameters for updating a user.
        :return: the updated user object :class:`yagocd.resources.user.UserEntity`.
        :rtype: yagocd.resources.user.UserEntity
        """
        response = self._session.patch(
            path='{base_api}/users/{login}'.format(
                base_api=self.base_api,
                login=login
            ),
            data=json.dumps(options),
            headers={
                'Content-Type': 'application/json'
            }
        )

        return UserEntity(session=self._session, data=response.json())

    def delete(self, login):
        """
        Deletes a user.
        :param login: login name.
        :return: a message confirmation if the user was deleted.
        """
        response = self._session.delete(
            path='{base_api}/users/{login}'.format(
                base_api=self.base_api,
                login=login
            ),
        )

        return response.json().get('message')


class UserEntity(Base):
    """
    The user object.
    """
    pass


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.client import Yagocd
from yagocd.session import Session
from yagocd.resources import feed

import pytest


class BaseTestConfigurationManager(object):
    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def manager(self, session):
        return feed.FeedManager(session=session)


class TestPipelines(BaseTestConfigurationManager):
    def test_pipelines_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipelines") as cass:
            manager.pipelines()
            assert cass.requests[0].path == '/go/api/pipelines.xml'

    def test_pipelines_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipelines") as cass:
            manager.pipelines()
            assert cass.requests[0].method == 'GET'

    def test_pipelines_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipelines") as cass:
            manager.pipelines()
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_pipelines_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipelines") as cass:
            manager.pipelines()
            assert cass.responses[0]['status']['code'] == 200

    def test_pipelines_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipelines"):
            result = manager.pipelines()
            assert isinstance(result, basestring)


class TestPipelineById(BaseTestConfigurationManager):
    PIPELINE_ID = 3

    def test_pipeline_by_id_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipeline_by_id") as cass:
            manager.pipeline_by_id(self.PIPELINE_ID)
            assert cass.requests[0].path == '/go/api/pipelines/THIS_PARAMETER_IS_USELESS/3.xml'

    def test_pipeline_by_id_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipeline_by_id") as cass:
            manager.pipeline_by_id(self.PIPELINE_ID)
            assert cass.requests[0].method == 'GET'

    def test_pipeline_by_id_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipeline_by_id") as cass:
            manager.pipeline_by_id(self.PIPELINE_ID)
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_pipeline_by_id_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipeline_by_id") as cass:
            manager.pipeline_by_id(self.PIPELINE_ID)
            assert cass.responses[0]['status']['code'] == 200

    def test_pipeline_by_id_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/pipeline_by_id"):
            result = manager.pipeline_by_id(self.PIPELINE_ID)
            assert isinstance(result, basestring)


class TestStages(BaseTestConfigurationManager):
    PIPELINE_NAME = 'Shared_Services'

    def test_stages_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stages") as cass:
            manager.stages(self.PIPELINE_NAME)
            assert cass.requests[0].path == '/go/api/pipelines/{}/stages.xml'.format(self.PIPELINE_NAME)

    def test_stages_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stages") as cass:
            manager.stages(self.PIPELINE_NAME)
            assert cass.requests[0].method == 'GET'

    def test_stages_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stages") as cass:
            manager.stages(self.PIPELINE_NAME)
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_stages_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stages") as cass:
            manager.stages(self.PIPELINE_NAME)
            assert cass.responses[0]['status']['code'] == 200

    def test_stages_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stages"):
            result = manager.stages(self.PIPELINE_NAME)
            assert isinstance(result, basestring)


class TestStageById(BaseTestConfigurationManager):
    STAGE_ID = 11

    def test_stage_by_id_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage_by_id") as cass:
            manager.stage_by_id(self.STAGE_ID)
            assert cass.requests[0].path == '/go/api/stages/{}.xml'.format(self.STAGE_ID)

    def test_stage_by_id_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage_by_id") as cass:
            manager.stage_by_id(self.STAGE_ID)
            assert cass.requests[0].method == 'GET'

    def test_stage_by_id_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage_by_id") as cass:
            manager.stage_by_id(self.STAGE_ID)
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_stage_by_id_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage_by_id") as cass:
            manager.stage_by_id(self.STAGE_ID)
            assert cass.responses[0]['status']['code'] == 200

    def test_stage_by_id_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage_by_id"):
            result = manager.stage_by_id(self.STAGE_ID)
            assert isinstance(result, basestring)


class TestStage(BaseTestConfigurationManager):
    PIPELINE_NAME = 'Shared_Services'
    PIPELINE_COUNTER = '6'
    STAGE_NAME = 'Commit'
    STAGE_COUNTER = 1

    def test_stage_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage") as cass:
            manager.stage(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER
            )
            assert cass.requests[0].path == '/go/pipelines/{}/{}/{}/{}.xml'.format(
                self.PIPELINE_NAME, self.PIPELINE_COUNTER, self.STAGE_NAME, self.STAGE_COUNTER
            )

    def test_stage_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage") as cass:
            manager.stage(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER
            )
            assert cass.requests[0].method == 'GET'

    def test_stage_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage") as cass:
            manager.stage(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER
            )
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_stage_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage") as cass:
            manager.stage(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER
            )
            # we've been redirected to stage-id url
            assert cass.responses[0]['status']['code'] == 302

    def test_stage_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/stage"):
            result = manager.stage(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER
            )
            assert isinstance(result, basestring)


class TestJobStageById(BaseTestConfigurationManager):
    JOB_ID = 1

    def test_job_by_id_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/job_by_id") as cass:
            manager.job_by_id(self.JOB_ID)
            assert cass.requests[0].path == '/go/api/jobs/{}.xml'.format(self.JOB_ID)

    def test_job_by_id_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/job_by_id") as cass:
            manager.job_by_id(self.JOB_ID)
            assert cass.requests[0].method == 'GET'

    def test_job_by_id_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/job_by_id") as cass:
            manager.job_by_id(self.JOB_ID)
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_job_by_id_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/job_by_id") as cass:
            manager.job_by_id(self.JOB_ID)
            assert cass.responses[0]['status']['code'] == 200

    def test_job_by_id_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("feed/job_by_id"):
            result = manager.job_by_id(self.JOB_ID)
            assert isinstance(result, basestring)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'six',
    'requests',
    'easydict'

]

test_requirements = [
    # TODO: put package test requirements here
]

packages = ['yagocd']
for dirpath, dirname, filenames in os.walk('yagocd'):
    packages.append('.'.join(dirpath.split(os.sep)))

setup(
    name='yagocd',
    version='0.1.7',
    description="Yet another Python client for ThoughtWorks GOCD REST API.",
    long_description=readme + '\n\n' + history,
    author="Grigory Chernyshev",
    author_email='systray@yandex.ru',
    url='https://github.com/grundic/yagocd',
    packages=packages,
    package_dir={'yagocd': 'yagocd'},
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    keywords='yagocd',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
